package PScm::Primitive;

use strict;
use warnings;
use base qw(PScm::Expr);

sub Apply {
    my ($self, $form) = @_;

    my @unevaluated_args = $form->value;
    my @evaluated_args = map { $_->Eval() } @unevaluated_args;
    return $self->_apply(@evaluated_args);
}

sub _check_type {
    my ($self, $thing, $type) = @_;

    die "wrong type argument(", ref($thing), ") to ", ref($self),
      "\n"
      unless $thing->isa($type);
}

##################################
package PScm::Primitive::Multiply;

use base qw(PScm::Primitive);

sub _apply {
    my ($self, @args) = @_;

    my $result = PScm::Expr::Number->new(1)->value();

    while (@args) {
        my $arg = shift @args;
        $self->_check_type($arg, 'PScm::Expr::Number');
        $result *= $arg->value;
    }

    return new PScm::Expr::Number($result);
}

##################################
package PScm::Primitive::Subtract;

use base qw(PScm::Primitive);

sub _apply {
    my ($self, @args) = @_;

    unshift @args, PScm::Expr::Number->new(0) if @args < 2;

    my $arg = shift @args;
    $self->_check_type($arg, 'PScm::Expr::Number');

    my $result = $arg->value;

    while (@args) {
        $arg = shift @args;
        $self->_check_type($arg, 'PScm::Expr::Number');
        $result -= $arg->value;
    }

    return new PScm::Expr::Number($result);
}

1;
